<?php

namespace Drupal\addtocal_augment\Plugin\DateAugmenter;

use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\PluginFormInterface;
use Drupal\date_augmenter\DateAugmenter\DateAugmenterPluginBase;
use Drupal\date_augmenter\Plugin\PluginFormTrait;

/**
 * Date Augmenter plugin to inject Add to Calendar links.
 *
 * @DateAugmenter(
 *   id = "addtocal",
 *   label = @Translation("Add To Calendar Links"),
 *   description = @Translation("Adds links to add an events dates to a user's preferred calendar."),
 *   weight = 0
 * )
 */
class AddToCal extends DateAugmenterPluginBase implements PluginFormInterface {

  use PluginFormTrait;

  protected $processService;
  protected $config;
  protected $output;

  // /**
  //  * {@inheritdoc}
  //  */
  // public function settings($setting_key = NULL) {
  //   if (!is_null($setting_key)) {
  //     return $this->pluginDefinition['settings'][$setting_key] ?? $this->pluginDefinition['settings'];
  //   }
  //   return $this->pluginDefinition['settings'];
  // }

  /**
   * Builds and returns a render array for the task.
   *
   * @param array $output
   *   The existing render array, to be augmented, passed by reference.
   * @param Drupal\Core\Datetime\DrupalDateTime $start
   *   The object which contains the start time.
   * @param Drupal\Core\Datetime\DrupalDateTime $end
   *   The optionalobject which contains the end time.
   * @param array $options
   *   An array of options to further guide output.
   */
  public function augmentOutput(array &$output, DrupalDateTime $start, DrupalDateTime $end = NULL, array $options = []) {
    // Do stuff.
    $config = $this->getConfiguration();
    if (empty($config['title']) && !isset($options['entity'])) {
      // TODO: log some kind of warning that we can't work without the entity
      // or a provided title?
      return;
    }
    $end_fallback = $end ?? $start;
    $now = new DrupalDateTIme();
    if ($end_fallback < $now && !$config['past_evants']) {
      return;
    }
    $entity = $options['entity'];
    if (!$end) {
      $end = $start;
    }
    // TODO: retrieve config
    $config = $options['config'] ?? [];
    $timezone = $options['timezone'] ?? NULL;
    $ical_link = 'data:text/calendar;charset=utf8,BEGIN:VCALENDAR%0AVERSION:2.0%0ABEGIN:VEVENT'
                 // TODO: Format with timezone if set, instead of UTC?
                 . '%0ADTSTART:' . $start->format("Ymd\\THi00\\Z", $timezone)
                 . '%0ADTEND:' . $end->format("Ymd\\THi00\\Z", $timezone)
                 . '%0ASUMMARY:' . $entity->label();
    if (isset($config['description'])) {
      // TODO: strip tags, etc.
      $ical_link .= '%0ADESCRIPTION:' . $entity->get($config['description'])->value;
    }
    if (isset($config['location'])) {
      // TODO: strip tags, etc.
      // TODO: enable the use of tokens.
      $ical_link .= '%0ALOCATION:' . $entity->get($config['location'])->value;
    }
    $ical_link .= '%0AEND:VEVENT%0AEND:VCALENDAR';

    $google_link = 'https://www.google.com/calendar/r/eventedit?text='
                   . $entity->label()
                   . '&dates=' . $start->format("Ymd\\THi00\\Z", $timezone)
                   . '/' . $end->format("Ymd\\THi00\\Z", $timezone);
    if (isset($config['description'])) {
      // TODO: strip tags, line breaks, non-breaking spaces, extra spaces, etc.
      // if (!$entity->get('body')->isEmpty()) {
      //   // Strip tags.
      //   $output = strip_tags(str_replace('<', ' <', $entity->get('body')->getValue()[0]['value']));

      //   // Strip out line breaks.
      //   $output = preg_replace('/\n|\r|\t/m', ' ', $output);

      //   // Strip out non-breaking spaces.
      //   $output = str_replace('&nbsp;', ' ', $output);
      //   $output = str_replace("\xc2\xa0", ' ', $output);

      //   // Strip out extra spaces.
      //   $output = trim(preg_replace('/\s\s+/', ' ', $output));
      //   $truncate = new TruncateHTML();
      //   $output = $truncate->truncateChars($output, 60, '...');
      // }

      $google_link .= '&details=' . $entity->get($config['description'])->value;
    }
    if (isset($config['location'])) {
      // TODO: strip tags, etc.
      $google_link .= '&location=' . $entity->get($config['location'])->value;
    }

    $output['addtocal'] = [
      '#theme' => 'addtocal_links',
      '#google' => $google_link,
      '#ical' => $ical_link,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'event_title' => '',
      'location' => '',
      'description' => '',
      'past_events' => FALSE,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['event_title'] = [
      '#title' => $this->t('Event title'),
      '#type' => 'textfield',
      '#default_value' => $this->configuration['event_title'],
      '#description' => $this->t('Optional - if left empty, the entity label will be used. You can use static text or tokens.'),
    ];

    $form['location'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Location'),
      '#description' => $this->t('Optional. You can use static text or tokens.'),
      '#default_value' => $this->configuration['location'],
    ];

    $form['description'] = [
      '#title' => $this->t('Event description'),
      '#type' => 'textarea',
      '#default_value' => $this->configuration['description'],
      '#description' => $this->t('Optional. You can use static text or tokens.'),
    ];

    $form['past_events'] = [
      '#title' => $this->t('Show Add to Cal widget for past events?'),
      '#type' => 'checkbox',
      '#default_value' => $this->configuration['past_events'],
    ];

    // $form['token_tree_link'] = [
    //   '#theme' => 'token_tree_link',
    //   '#token_types' => [
    //     $field->getTargetEntityTypeId(),
    //   ],
    // ];

    return $form;
  }

}
